package com.streambase.sb.contrib.client.jmx;

import java.util.LinkedHashMap;
import java.util.Set;


public class JMXCommands {
    
	public class ParamSig {
    	
    	public Object[] param;
    	public final String[] signature;
    	public final String mBeanName;
    	public final String helpText;
    	
    	ParamSig(Object[] params, String[] signature, String mBeanName, String helpText) {
    		this.param=params;
    		this.signature=signature;
    		this.mBeanName=mBeanName;
    		this.helpText=helpText;
    	}
    }
    
    private LinkedHashMap<String, ParamSig> commandMap=new LinkedHashMap<String, ParamSig>();
    
    public JMXCommands() {
    	init();
    }
	
    public Set<String> getCommands() {
    	return commandMap.keySet();
    }
    
    private void init() {

		commandMap.put("getBusyOperators", new ParamSig(null,
				new String[] { "java.lang.Integer" },
				SBJMX.STREAMBASE_JMX_PROFILE_NAME,
				"Top - Returns a list of Top X operators with the highest CPU accumulation"));
		commandMap.put("getDynamicVariable", new ParamSig(null,
				new String[] { "java.lang.String" },
				SBJMX.STREAMBASE_JMX_MANAGEMENT_NAME,
				"myDynamicVariableName - Returns a String value of myDynamicVariableName"));
		commandMap.put("getOperatorProperties", new ParamSig(null,
				new String[] { "[Ljava.lang.String;" },
				SBJMX.STREAMBASE_JMX_MANAGEMENT_NAME,
				"container operator - returns a String[] listing each property with its current value"));
		commandMap.put("getOperatorProperty", new ParamSig(null,
				new String[] { "[Ljava.lang.String;" },
				SBJMX.STREAMBASE_JMX_MANAGEMENT_NAME,
				"container operator property - returns a String with the value of the property"));
		commandMap.put("getOperatorProfile", new ParamSig(null,
				new String[] { "java.lang.String" },
				SBJMX.STREAMBASE_JMX_PROFILE_NAME,
				"OperName - Return a long array with input tuples, output tuples, Size, Time (in microseconds) for OperName"));
		commandMap.put("internalCommand", new ParamSig(null,
				new String[] { "[Ljava.lang.String;" },
				SBJMX.STREAMBASE_JMX_MANAGEMENT_NAME,
				"command - For internal use only"));
		commandMap.put("killConnection", new ParamSig(null,
				new String[] { "java.lang.String" },
				SBJMX.STREAMBASE_JMX_MANAGEMENT_NAME,
				"ConnectionId - returns a status line for the closed connection"));
		commandMap.put("leadershipStatus", new ParamSig(null,
				new String[] { "java.lang.String" },
				SBJMX.STREAMBASE_JMX_MANAGEMENT_NAME,
				"[NON_]LEADER - sets the leadership status of server"));
		commandMap.put("listEntities", new ParamSig(
				new Object[] { new String[] { "default", "operator", "0" } },
				new String[] { "[Ljava.lang.String;" },
				SBJMX.STREAMBASE_JMX_MANAGEMENT_NAME, 
				"[container entityType flags] - returns a String [] with the entity listing"));
		commandMap.put("listOperators", new ParamSig(null, null,
				SBJMX.STREAMBASE_JMX_PROFILE_NAME,
				"- List all operators in the server with input tuples, output tuples, Size, Time (in microseconds) for each"));
		commandMap.put("listQueues", new ParamSig(null, null,
				SBJMX.STREAMBASE_JMX_PROFILE_NAME,
				"- List all queues in the server with max size, last size, ave uS latency, batch time in uS"));
		commandMap.put("listStreams", new ParamSig(null, null,
				SBJMX.STREAMBASE_JMX_PROFILE_NAME,
				"- List all streams in the server with the total number of tuples enqueued or dequeued"));
		commandMap.put("restartOperator", new ParamSig(null,
				new String[] { "[Ljava.lang.String;" },
				SBJMX.STREAMBASE_JMX_MANAGEMENT_NAME,
				"operatorName - restart the operator"));
		commandMap.put("resume", new ParamSig(null,
				new String[] { "java.lang.String" },
				SBJMX.STREAMBASE_JMX_MANAGEMENT_NAME,
				"container - resumes the container"));
		commandMap.put("resumeOperator", new ParamSig(null,
				new String[] { "[Ljava.lang.String;" },
				SBJMX.STREAMBASE_JMX_MANAGEMENT_NAME,
				"operatorName - resume the operator"));
		commandMap.put("setDynamicVariable", new ParamSig(null,
				new String[] { "[Ljava.lang.String;" },
				SBJMX.STREAMBASE_JMX_MANAGEMENT_NAME,
				"myDynamicVariableName value - sets the dynamic variable"));
		commandMap.put("setOperatorProperty", new ParamSig(null,
				new String[] { "[Ljava.lang.String;" },
				SBJMX.STREAMBASE_JMX_MANAGEMENT_NAME,
				"container operator propterty value - sets the operator property value"));
		commandMap.put("setRuntimeParam", new ParamSig(null,
				new String[] { "[Ljava.lang.String;" },
				SBJMX.STREAMBASE_JMX_MANAGEMENT_NAME,
				"param value container - internal use only"));
		commandMap.put("shutdownOperator", new ParamSig(null,
				new String[] { "[Ljava.lang.String;" },
				SBJMX.STREAMBASE_JMX_MANAGEMENT_NAME,
				"operatorName - shutdown the operator"));
		commandMap.put("shutdown", new ParamSig(
				new Object[] { new String("") },
				new String[] { "java.lang.String" },
				SBJMX.STREAMBASE_JMX_MANAGEMENT_NAME,
				"[operator | container] - no arg, shutdown server"));
		commandMap.put("statusOperator", new ParamSig(null,
				new String[] { "java.lang.String" },
				SBJMX.STREAMBASE_JMX_MANAGEMENT_NAME,
				"container - return the status of all operators in the container"));
		commandMap.put("suspend", new ParamSig(null,
				new String[] { "java.lang.String" },
				SBJMX.STREAMBASE_JMX_MANAGEMENT_NAME,
				"operator | container - suspend the operator or container"));
		commandMap.put("suspendOperator", new ParamSig(null,
				new String[] { "[Ljava.lang.String;" },
				SBJMX.STREAMBASE_JMX_MANAGEMENT_NAME,
				"operatorName - suspends the operator"));

		// These are actually attributes, no operations
		commandMap.put("ListConnections", new ParamSig(
				new Object[] { new String("") },
				new String[] { "java.lang.String" },
				SBJMX.STREAMBASE_JMX_MANAGEMENT_NAME,
				"- lists the connections to the server"));
		commandMap.put("LeadershipStatus", new ParamSig(
				new Object[] { new String("") },
				new String[] { "java.lang.String" },
				SBJMX.STREAMBASE_JMX_MANAGEMENT_NAME,
				"- returns the leadership status of the server"));
		commandMap.put("Status", new ParamSig(new Object[] { new String("") },
				new String[] { "java.lang.String" },
				SBJMX.STREAMBASE_JMX_MANAGEMENT_NAME,
				"- returns a status string for the server"));

    }
    
    public ParamSig getCommand(String command) {
    	return commandMap.get(command);
    }
    
}
