package com.streambase.sb.contrib.client.jmx;

import java.io.IOException;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.LinkedHashSet;

import javax.management.MBeanAttributeInfo;
import javax.management.MBeanInfo;
import javax.management.MBeanOperationInfo;
import javax.management.MBeanServerConnection;
import javax.management.ObjectName;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;

import com.streambase.sb.contrib.client.jmx.JMXCommands.ParamSig;
import com.streambase.sb.util.Msg;
import com.streambase.sb.util.Util;

public class SBJMX {

	/**
	 * @param args
	 */
    public static final String STREAMBASE_JMX_MANAGEMENT_NAME=Util.getSystemProperty("streambase.server.jmx.name", "StreamBase:type=Server,name=Management"); 
    public static final String STREAMBASE_JMX_PROFILE_NAME=Util.getSystemProperty("streambase.server.jmx.name", "StreamBase:type=Server,name=Profiler"); 
    
    private static String commandName="sbjmx";
    
    private static void usage() {
    	output(commandName + " [-u uri | -p port] [-m MBean] command [ params ... ]\n");
    	// just dump the known commands
    	
    	output("    Where:");
    	output("    uri = service:jmx:rmi:///jndi/rmi://HostName:PortNumber/jmxrmi");
    	output("    port = port number for localhost uri");
    	output("    command is one of:\n");
    	JMXCommands jmxCmd = new JMXCommands();
    	for (String commandName : jmxCmd.getCommands()) {
    		String help=jmxCmd.getCommand(commandName).helpText;
    		output("    " + commandName + " " + help);
    	}

    	System.exit(-1);
    }
    
	public static void main(String[] args) {
		
		final String PORT_ARG="-p";
		final String URI_ARG="-u";
		final String MBEAN_ARG="-m";
		final String LOOP_ARG="-l";
		final String HELP1="-h";
		final String HELP2="--help";
		final String HELP3="-?";
		
		LinkedHashSet<String> otherOpts = new LinkedHashSet<String>();
		int port=0;
		String uri=null;
		String mBeanName=null;
		int loopCount=0;
		
		for (int i=0; i<args.length; i++) {
			if (PORT_ARG.equals(args[i])) {
				port=Integer.parseInt(args[++i]);
				continue;
			}
			if (URI_ARG.equals(args[i])) {
				uri=args[++i];
				continue;
			}
			if (MBEAN_ARG.equals(args[i])) {
				mBeanName=args[++i];
				continue;
			}
			if (LOOP_ARG.equals(args[i])) {
				loopCount= Integer.parseInt(args[++i]);
				continue;
			}
			if (HELP1.equals(args[i]) || HELP2.equals(args[i]) || HELP3.equals(args[i]) ) {
				usage();
			}
			otherOpts.add(args[i]);
		}

		if (Util.isEmpty(uri) && port==0) {
			Msg.notice("You must supply a port or URI");
			usage();
		}

		if (otherOpts.size() < 1) {
			Msg.notice("You must supply a command");
			usage();
		}
		
		Iterator<String> it=otherOpts.iterator();
		String myCommand=it.next();
		it.remove();
		
		String[] otherArgs=null;
		if (otherOpts.size() > 0) {
			otherArgs=new String[otherOpts.size()];
			otherOpts.toArray(otherArgs);
		}
		
		if (Util.isEmpty(myCommand)) {
			Msg.notice("You must supply a command");
			usage();
		}
		
	    JMXServiceURL url=null;  
		try {
			// uri should look like:
			// "service:jmx:rmi:///jndi/rmi://:9999/jmxrmi"
			// or just a port number...
		    if (Util.isEmpty(uri)) {
		    	uri = "service:jmx:rmi:///jndi/rmi://:" + port + "/jmxrmi";
		    }			
			url = new JMXServiceURL(uri);
		} catch (MalformedURLException e) {
			e.printStackTrace();
			usage();
		}
		JMXConnector jmxc=null;
	    try {
			jmxc = JMXConnectorFactory.connect(url, null);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.exit(-1);
		}
	
		MBeanServerConnection mbsc=null;
		 try {
			mbsc = jmxc.getMBeanServerConnection();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.exit(-1);
		}
	
		String domains[]=null;
		try {
			domains = mbsc.getDomains();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.exit(-1);
		}
	
		boolean found=false;
		for (int i=0; i < domains.length; i++) {
			if ("StreamBase".equals(domains[i])) {
				found=true;
				break;
			}
		}
		
		if (!found) {
			System.out.println("StreamBase Management domain not found");
			System.exit(-1);
		}
	
		JMXCommands jmxCmd = new JMXCommands();
		
		// Get the command signature
		ParamSig paramSig=jmxCmd.getCommand(myCommand);
		if (paramSig == null) {
			System.out.println("This implemenation doesn't know: " + myCommand);
			usage();
		}
		if (Util.isEmpty(mBeanName)) {
			mBeanName=paramSig.mBeanName;
		}
		
		MBeanInfo sbMBean=null;
		ObjectName on=null;
		try {
			on=new ObjectName(mBeanName);
			
			sbMBean=mbsc.getMBeanInfo(on);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.exit(-1);
		} 
		
		if (sbMBean == null) {
			System.out.println("The remote JVM doesn't know: " + mBeanName);
			System.exit(-1);
		}
		
		// Get the command bean from the server
		MBeanOperationInfo[] sbInfo=sbMBean.getOperations();
		MBeanOperationInfo mBean=null;
		for (int i=0; i < sbInfo.length; i++) {
			if (sbInfo[i].getName().equals(myCommand)) {
				mBean=sbInfo[i];
				break;
			}
		}
		
		MBeanAttributeInfo mAttrib=null;
		if (mBean == null) {
			MBeanAttributeInfo[] sbAttrib=sbMBean.getAttributes();
			for (int i=0; i < sbAttrib.length; i++) {
				if (sbAttrib[i].getName().equals(myCommand)) {
					mAttrib=sbAttrib[i];
					break;
				}
			}
			if (mAttrib ==null) {
				System.out.println("The server doesn't know: " + myCommand);
				System.exit(-1);
			}
		}
		
		// see if there are parameters to fill in
		if (otherArgs != null) {
			if (paramSig.signature[0].equals("[Ljava.lang.String;")) {
				Object[] opObs= new Object[] { otherArgs };
				paramSig.param = opObs; 
			} else if (paramSig.signature[0].equals("java.lang.String")) {
				paramSig.param = new Object[] { otherArgs[0]};
			} else if (paramSig.signature[0].equals("java.lang.Integer")) {
				paramSig.param = new Object[] { Integer.parseInt(otherArgs[0])};
			}
		}
		
		do {
			// Do the command
			Object ret=null;
			try {
				if (mBean != null) {
					ret=mbsc.invoke(on, mBean.getName(), paramSig.param, paramSig.signature);
				} else {
					ret=mbsc.getAttribute(on, mAttrib.getName());
				}
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				System.exit(-1);
			}
			
			if (ret == null) {
				output("No Output returned");
			} else {
				display(ret) ;
			}
		} while (--loopCount > 0);
	}
	
	private static void display(Object o) {

		if (o instanceof String) {
			System.out.println(o);
			return;
		}
		
		if (o instanceof String[]) {
			String[] sArray=(String [])o;
			for (int i=0; i < sArray.length; i++) {
				System.out.println(sArray[i]);
			}
			return;
		}

		if (o instanceof ArrayList<?>) {
			@SuppressWarnings("unchecked")
			ArrayList<String> sArray=(ArrayList<String>)o;
			for (int i=0; i < sArray.size(); i++) {
				System.out.println(sArray.get(i));
			}
			return;
		}
		
		if (o.getClass().isArray()) {
			String[] sArray;
			if (o instanceof long[]) {
				long[] lArray=(long[])o;
				for (int i=0; i < lArray.length; i++) {
					System.out.println(lArray[i]);
				}				
			} else {
				sArray=(String [])o;
				for (int i=0; i < sArray.length; i++) {
					System.out.println(sArray[i]);
				}
			}
			return;
		}
		
		if (o instanceof Arrays) {
			String[] sArray=(String [])o;
			for (int i=0; i < sArray.length; i++) {
				System.out.println(sArray[i]);
			}
			return;
		}
		
		System.out.println(o.toString());
		
	}
	
	private static void output(String s) {
		System.out.println(s);
	}
}
